# Copyright 2015 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


require "delegate"

module Google
  module Cloud
    module ResourceManager
      class Project
        ##
        # Project::List is a special case Array with additional values.
        class List < DelegateClass(::Array)
          ##
          # If not empty, indicates that there are more projects that match
          # the request and this value should be passed to continue.
          attr_accessor :token

          ##
          # @private Create a new Project::List with an array of Project
          # instances.
          def initialize arr = []
            super arr
          end

          ##
          # Whether there is a next page of projects.
          #
          # @return [Boolean]
          #
          # @example
          #   require "google/cloud/resource_manager"
          #
          #   resource_manager = Google::Cloud::ResourceManager.new
          #
          #   projects = resource_manager.projects
          #   if projects.next?
          #     next_projects = projects.next
          #   end
          #
          def next?
            !token.nil?
          end

          ##
          # Retrieve the next page of projects.
          #
          # @return [Project::List]
          #
          # @example
          #   require "google/cloud/resource_manager"
          #
          #   resource_manager = Google::Cloud::ResourceManager.new
          #
          #   projects = resource_manager.projects
          #   if projects.next?
          #     next_projects = projects.next
          #   end
          #
          def next
            return nil unless next?
            ensure_manager!
            @manager.projects token: token, filter: @filter, max: @max
          end

          ##
          # Retrieves remaining results by repeatedly invoking {#next} until
          # {#next?} returns `false`. Calls the given block once for each
          # result, which is passed as the argument to the block.
          #
          # An Enumerator is returned if no block is given.
          #
          # This method will make repeated API calls until all remaining results
          # are retrieved. (Unlike `#each`, for example, which merely iterates
          # over the results returned by a single API call.) Use with caution.
          #
          # @param [Integer] request_limit The upper limit of API requests to
          #   make to load all projects. Default is no limit.
          # @yield [project] The block for accessing each project.
          # @yieldparam [Project] project The project object.
          #
          # @return [Enumerator]
          #
          # @example Iterating each project by passing a block:
          #   require "google/cloud/resource_manager"
          #
          #   resource_manager = Google::Cloud::ResourceManager.new
          #   projects = resource_manager.projects
          #
          #   projects.all do |project|
          #     puts project.project_id
          #   end
          #
          # @example Using the enumerator by not passing a block:
          #   require "google/cloud/resource_manager"
          #
          #   resource_manager = Google::Cloud::ResourceManager.new
          #   projects = resource_manager.projects
          #
          #   all_project_ids = projects.all.map do |project|
          #     project.project_id
          #   end
          #
          # @example Limit the number of API calls made:
          #   require "google/cloud/resource_manager"
          #
          #   resource_manager = Google::Cloud::ResourceManager.new
          #   projects = resource_manager.projects
          #
          #   projects.all(request_limit: 10) do |project|
          #     puts project.project_id
          #   end
          #
          def all request_limit: nil, &block
            request_limit = request_limit.to_i if request_limit
            unless block_given?
              return enum_for :all, request_limit: request_limit
            end
            results = self
            loop do
              results.each(&block)
              if request_limit
                request_limit -= 1
                break if request_limit.negative?
              end
              break unless results.next?
              results = results.next
            end
          end

          ##
          # @private New Projects::List from a response object.
          def self.from_gapi gapi_list, manager, filter = nil, max = nil
            projects = new(Array(gapi_list.projects).map do |gapi_object|
              Project.from_gapi gapi_object, manager.service
            end)
            projects.instance_variable_set :@token,   gapi_list.next_page_token
            projects.instance_variable_set :@manager, manager
            projects.instance_variable_set :@filter,  filter
            projects.instance_variable_set :@max,     max
            projects
          end

          protected

          ##
          # Raise an error unless an active connection is available.
          def ensure_manager!
            raise "Must have active connection" unless @manager
          end
        end
      end
    end
  end
end
